"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.duplicate = exports.update = void 0;
const actions_1 = require("@nocobase/actions");
const database_1 = require("@nocobase/database");
function update(context, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const { db } = context;
        const repository = actions_1.utils.getRepositoryFromParams(context);
        const { filterByTk, values, whitelist, blacklist, filter, updateAssociationValues } = context.action.params;
        context.body = yield db.sequelize.transaction((transaction) => __awaiter(this, void 0, void 0, function* () {
            const others = {};
            if (values.enabled) {
                values.current = true;
                others.enabled = false;
            }
            if (values.current) {
                others.current = false;
                yield repository.update({
                    filter: {
                        key: values.key,
                        id: {
                            [database_1.Op.ne]: filterByTk
                        }
                    },
                    values: others,
                    context,
                    transaction
                });
            }
            const instance = yield repository.update({
                filterByTk,
                values,
                whitelist,
                blacklist,
                filter,
                updateAssociationValues,
                context,
                transaction
            });
            return instance;
        }));
        yield next();
    });
}
exports.update = update;
function typeOf(value) {
    if (Array.isArray(value)) {
        return 'array';
    }
    else if (value instanceof Date) {
        return 'date';
    }
    else if (value === null) {
        return 'null';
    }
    return typeof value;
}
function migrateConfig(config, oldToNew) {
    function migrate(value) {
        switch (typeOf(value)) {
            case 'object':
                return Object.keys(value).reduce((result, key) => (Object.assign(Object.assign({}, result), { [key]: migrate(value[key]) })), {});
            case 'array':
                return value.map(item => migrate(item));
            case 'string':
                return value
                    .replace(/(\{\{\$jobsMapByNodeId\.)(\d+)/, (_, prefix, id) => `${prefix}${oldToNew.get(Number.parseInt(id, 10)).id}`);
            default:
                return value;
        }
    }
    return migrate(config);
}
function duplicate(context, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const { db } = context;
        const repository = actions_1.utils.getRepositoryFromParams(context);
        const { filterByTk } = context.action.params;
        context.body = yield db.sequelize.transaction((transaction) => __awaiter(this, void 0, void 0, function* () {
            var e_1, _a, e_2, _b;
            var _c, _d;
            const origin = yield repository.findOne({
                filterByTk,
                appends: ['nodes'],
                context,
                transaction
            });
            const instance = yield repository.create({
                values: {
                    key: origin.key,
                    title: origin.title,
                    description: origin.description,
                    type: origin.type,
                    config: origin.config
                },
                transaction
            });
            const originalNodesMap = new Map();
            origin.nodes.forEach((node) => {
                originalNodesMap.set(node.id, node);
            });
            const oldToNew = new Map();
            const newToOld = new Map();
            try {
                for (var _e = __asyncValues(origin.nodes), _f; _f = yield _e.next(), !_f.done;) {
                    const node = _f.value;
                    const newNode = yield instance.createNode({
                        type: node.type,
                        config: node.config,
                        title: node.title,
                        branchIndex: node.branchIndex
                    }, { transaction });
                    // NOTE: keep original node references for later replacement
                    oldToNew.set(node.id, newNode);
                    newToOld.set(newNode.id, node);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_f && !_f.done && (_a = _e.return)) yield _a.call(_e);
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                for (var _g = __asyncValues(oldToNew.entries()), _h; _h = yield _g.next(), !_h.done;) {
                    const [oldId, newNode] = _h.value;
                    const oldNode = originalNodesMap.get(oldId);
                    const newUpstream = oldNode.upstreamId ? oldToNew.get(oldNode.upstreamId) : null;
                    const newDownstream = oldNode.downstreamId ? oldToNew.get(oldNode.downstreamId) : null;
                    yield newNode.update({
                        upstreamId: (_c = newUpstream === null || newUpstream === void 0 ? void 0 : newUpstream.id) !== null && _c !== void 0 ? _c : null,
                        downstreamId: (_d = newDownstream === null || newDownstream === void 0 ? void 0 : newDownstream.id) !== null && _d !== void 0 ? _d : null,
                        config: migrateConfig(oldNode.config, oldToNew)
                    }, { transaction });
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_h && !_h.done && (_b = _g.return)) yield _b.call(_g);
                }
                finally { if (e_2) throw e_2.error; }
            }
            return instance;
        }));
        yield next();
    });
}
exports.duplicate = duplicate;
//# sourceMappingURL=workflows.js.map